/*
	CDiag.h
		the C prototypes for the Diag routines
		last modified 8 Aug 15 th
*/


#ifndef __CDiag_h__
#define __CDiag_h__

#ifndef RealType
#define RealType double
#endif

#ifdef __cplusplus

#include <complex>
typedef std::complex<RealType> ComplexType;
#define Re(x) (x).real()
#define Im(x) (x).imag()
#define ToComplex(r,i) ComplexType(r, i)

extern "C" {

#else

#include <complex.h>
typedef RealType complex ComplexType;
#define Re(x) creal(x)
#define Im(x) cimag(x)
#define ToComplex(r,i) ((r) + (i)*I)

#endif

void HEigensystem(const int n,
  ComplexType *A, const int ldA,
  RealType *d,
  ComplexType *U, const int ldU, const int sort);

void SEigensystem(const int n,
  ComplexType *A, const int ldA,
  ComplexType *d,
  ComplexType *U, const int ldU, const int sort);

void CEigensystem(const int n,
  ComplexType *A, const int ldA,
  ComplexType *d,
  ComplexType *U, const int ldU, const int sort);

void TakagiFactor(const int n,
  ComplexType *A, const int ldA,
  RealType *d,
  ComplexType *U, const int ldU, const int sort);

void SVD(const int m, const int n,
  const ComplexType *A, const int ldA,
  RealType *d,
  ComplexType *V, const int ldV,
  ComplexType *W, const int ldW, const int sort);

extern int nsweeps;

#ifdef __cplusplus
}
#endif

#define Matrix(A) (ComplexType *)(A),sizeof((A)[0])/sizeof((A)[0][0])

#endif

