:Evaluate: BeginPackage["Diag`"]

:Evaluate: HEigensystem::usage =
	"HEigensystem[A] computes the eigenvalue decomposition {d, U} of the Hermitian matrix A.
	The results fulfill EQEIGEN with U unitary.
	HEigensystem[A, s] further determines the sorting:
	s = 1 ascending (default), -1 descending, 0 no sort."

:Evaluate: SEigensystem::usage =
	"SEigensystem[A] computes the eigenvalue decomposition {d, U} of the symmetric matrix A.
	The results fulfill EQEIGEN with Inverse[U] == Transpose[U].
	SEigensystem[A, s] further determines the sorting:
	s = 1 ascending (default), -1 descending, 0 no sort."

:Evaluate: CEigensystem::usage =
	"CEigensystem[A] computes the eigenvalue decomposition {d, U} of the general complex matrix A.
	The results fulfill EQEIGEN.
	CEigensystem[A, s] further determines the sorting:
	s = 1 ascending (default), -1 descending, 0 no sort."

:Evaluate: TakagiFactor::usage =
	"TakagiFactor[A] computes the Takagi factorization {d, U} of the symmetric matrix A.
	The results fulfill EQTAKAGI with U unitary.
	TakagiFactor[A, s] further determines the sorting:
	s = 1 ascending (default), -1 descending, 0 no sort."

:Evaluate: SVD::usage =
	"SVD[A] computes the Singular Value Decomposition {V, d, W} of the matrix A.
	The results fulfill EQSVD.
	SVD[A, s] further determines the sorting:
	s = 1 ascending (default), -1 descending, 0 no sort."

:Evaluate: $NSweeps::usage =
	"$NSweeps returns the number of sweeps needed to diagonalize the matrix in the last invocation of HEigensystem, SEigensystem, CEigensystem, TakagiFactor, or SVD."

:Evaluate: Begin["`Private`"]

:Evaluate: matQ[m_] := MatrixQ[m, NumericQ]

:Evaluate: sqmatQ[m_] := matQ[m] && SameQ@@ Dimensions[m]

:Evaluate: pack[m_] := Flatten @ Map[{Re[#], Im[#]}&, N[m], {2}]

:Evaluate: cplx[f_] := Chop @ Apply[Complex, Partition[f, 2], 1]

:Evaluate: unpack[f_, n_] := Partition[cplx[f], n]

:Begin:
:Function: mHEigensystem
:Pattern: HEigensystem[A_List?sqmatQ, sort_Integer:1]
:Arguments: {Length[A], pack[A], sort}
:ArgumentTypes: {Integer, RealList, Integer}
:ReturnType: Manual
:End:

:Begin:
:Function: mSEigensystem
:Pattern: SEigensystem[A_List?sqmatQ, sort_Integer:1]
:Arguments: {Length[A], pack[A], sort}
:ArgumentTypes: {Integer, RealList, Integer}
:ReturnType: Manual
:End:

:Begin:
:Function: mCEigensystem
:Pattern: CEigensystem[A_List?sqmatQ, sort_Integer:1]
:Arguments: {Length[A], pack[A], sort}
:ArgumentTypes: {Integer, RealList, Integer}
:ReturnType: Manual
:End:

:Begin:
:Function: mTakagiFactor
:Pattern: TakagiFactor[A_List?sqmatQ, sort_Integer:1]
:Arguments: {Length[A], pack[A], sort}
:ArgumentTypes: {Integer, RealList, Integer}
:ReturnType: Manual
:End:

:Begin:
:Function: mSVD
:Pattern: SVD[A_List?matQ, sort_Integer:1]
:Arguments: {Length[A], Length[A[[1]]], pack[A], sort}
:ArgumentTypes: {Integer, Integer, RealList, Integer}
:ReturnType: Manual
:End:

:Begin:
:Function: mnsweeps
:Pattern: $NSweeps
:Arguments: {}
:ArgumentTypes: {}
:ReturnType: Integer
:End:

:Evaluate: End[]

:Evaluate: EndPackage[]



/*
	Diag.tmc
		the Mathematica frontend for the libdiag routines
		last modified 21 Aug 15 th
*/


#include <stdlib.h>
#include <unistd.h>

#include "mathlink.h"
#ifndef MLCONST
#define MLCONST
#endif

#include "diag-c.h"

typedef MLCONST char cchar;

static inline int MLPutComplexList(MLINK mlp, cComplexType *x, cint n)
{
  MLPutFunction(mlp, "Diag`Private`cplx", 1);
  return MLPutRealList(mlp, (RealType *)x, 2*n);
}

static inline int MLPutComplexMatrix(MLINK mlp, cComplexType *x, cint m, cint n)
{
  MLPutFunction(mlp, "Diag`Private`unpack", 2);
  MLPutRealList(mlp, (RealType *)x, 2*m*n);
  return MLPutInteger(mlp, n);
}

/******************************************************************/

#include "HEigensystem.c"

static void mHEigensystem(int n, RealType *Alist, int Alen, int sort)
{
  ComplexType A[n*n], U[n*n];
  RealType d[n];

  memcpy(A, Alist, Alen*sizeof(RealType));

  HEigensystem(n, A,n, d, U,n, sort);

  MLPutFunction(stdlink, "List", 2);
  MLPutRealList(stdlink, d, n);
  MLPutComplexMatrix(stdlink, U, n,n);

  MLEndPacket(stdlink);
}

/******************************************************************/

#include "SEigensystem.c"

static void mSEigensystem(int n, RealType *Alist, int Alen, int sort)
{
  ComplexType A[n*n], U[n*n], d[n];

  memcpy(A, Alist, Alen*sizeof(RealType));

  SEigensystem(n, A,n, d, U,n, sort);

  MLPutFunction(stdlink, "List", 2);
  MLPutComplexList(stdlink, d, n);
  MLPutComplexMatrix(stdlink, U, n,n);

  MLEndPacket(stdlink);
}

/******************************************************************/

#include "CEigensystem.c"

static void mCEigensystem(int n, RealType *Alist, int Alen, int sort)
{
  ComplexType A[n*n], U[n*n], d[n];

  memcpy(A, Alist, Alen*sizeof(RealType));

  CEigensystem(n, A,n, d, U,n, sort);

  MLPutFunction(stdlink, "List", 2);
  MLPutComplexList(stdlink, d, n);
  MLPutComplexMatrix(stdlink, U, n,n);

  MLEndPacket(stdlink);
}

/******************************************************************/

#include "TakagiFactor.c"

static void mTakagiFactor(int n, RealType *Alist, int Alen, int sort)
{
  ComplexType A[n*n], U[n*n];
  RealType d[n];

  memcpy(A, Alist, Alen*sizeof(RealType));

  TakagiFactor(n, A,n, d, U,n, sort);

  MLPutFunction(stdlink, "List", 2);
  MLPutRealList(stdlink, d, n);
  MLPutComplexMatrix(stdlink, U, n,n);

  MLEndPacket(stdlink);
}

/******************************************************************/

#include "SVD.c"

static void mSVD(int m, int n, RealType *Alist, int Alen, int sort)
{
  cint nm = (m < n) ? m : n;
  ComplexType V[nm*m], W[nm*n];
  RealType d[nm];

  SVD(m, n, (ComplexType *)Alist,n, d, V,m, W,n, sort);

  MLPutFunction(stdlink, "List", 3);
  MLPutComplexMatrix(stdlink, V, nm,m);
  MLPutRealList(stdlink, d, nm);
  MLPutComplexMatrix(stdlink, W, nm,n);

  MLEndPacket(stdlink);
}

/******************************************************************/

static int mnsweeps(void)
{
  return nsweeps;
}

/******************************************************************/

int main(int argc, char **argv)
{
  return MLMain(argc, argv);
}

