* demo-fortran.F
* a program demonstrating the use of the Diag library in Fortran
* this file is part of the Diag library
* last modified 19 Mar 10 th

* Notes:
*
* 1) SetMatrix has to be called every time because the
*    diagonalization routines overwrite the contents of the
*    matrix A.
*
* 2) HEigensystem, SEigensystem, and TakagiFactor assume they
*    are operating on a symmetric matrix and hence ignore the
*    below-diagonal entries.  Similarly, HEigensystem assumes
*    an Hermitian matrix and thus real diagonal entries.


#define dim 3

************************************************************************

	program DiagDemo
	implicit none

	integer n, sort
	parameter (n = dim, sort = 1)

	double complex A(n,n), U(n,n), V(n,n)
	double complex dc(n)
	double precision dr(n)
	integer i

100	format(/A, dim(/"d(", I1, ") = ", F10.6))
101	format(/A, dim(/"d(", I1, ") = ", F10.6, SP, F10.6, SS, " I"))

	call SetMatrix(n, A,n)
	call HEigensystem(n, A,n, dr, U,n, sort)
	print 100, "HEigensystem:", (i, dr(i), i = 1, n)

	call SetMatrix(n, A,n)
	call SEigensystem(n, A,n, dc, U,n, sort)
	print 101, "SEigensystem:", (i, dc(i), i = 1, n)

	call SetMatrix(n, A,n)
	call CEigensystem(n, A,n, dc, U,n, sort)
	print 101, "CEigensystem:", (i, dc(i), i = 1, n)

	call SetMatrix(n, A,n)
	call TakagiFactor(n, A,n, dr, U,n, sort)
	print 100, "TakagiFactor:", (i, dr(i), i = 1, n)

	call SetMatrix(n, A,n)
	call SVD(n,n, A,n, dr, U,n, V,n, sort)
	print 100, "SVD:", (i, dr(i), i = 1, n)
	end

************************************************************************

	subroutine SetMatrix(n, A,ldA)
	implicit none
	integer n, ldA
	double complex A(ldA,*)

	integer i, j

* put your matrix definition here:
	do j = 1, n
	  do i = 1, n
	    A(i,j) = DCMPLX(i, j)
	  enddo
	enddo
	end

